<?php
/*
Plugin Name: Link Verifier
Description: Verifies whether or not each link entered into the post editing area is valid.  
Author: Jake Kasprzak 
Version: 1.0
Author URI: http://jake.kasprzak.ca
*/

//link_verifier: given the text in the post editing area, modify the text so that broken links contain the HTTP error codes that would appear after clicking on them 
function link_verifier($text) {

	//go through each link in the post editing area, extract URL information on each one, then pass these URLs to function that verifies them
	//if HTTP error code (400 or greater) or a code of 0 returned, add that code to the end of the link    
	
	//initialize regular expressions for finding links and for extracting URL data in them
	
	$find_link_expression = '/<a(.*)<\/a>/i';

	$extract_url_expression = '/href="([^"]+)/';
	
	
	//initialize the first part of what is to be added to links to pages that would return error codes 
	$error_text = '<i>(Error Code:';

	//go through each link found, extract any URL data in it then add error code found by other function
	$num_links = preg_match_all($find_link_expression, $text, $link_array);
	
	for ($i=0; $i<$num_links; $i++) {
		
		//reset variable for text used to replace broken links
		$text_replace = "";
		
		$num_urls = preg_match($extract_url_expression, $link_array[0][$i], $the_url);
		
		if ($num_urls > 0) {
		
			$result = check_url($the_url[1]); 

			if ($result >= 400 || $result == 0) {

				//first part of replacement string: remove the last four characters from link,  which is the closing </a> tag (it will added again in the string that replaces this)
				$text_replace = substr($link_array[0][$i], 0, strlen($link_array[0][$i])-4);

				//complete this string will replace the link in the text
				$text_replace =  $text_replace . $error_text . $result . ")</i></a>";

				//create regular expression for finding the correct links 
				$new_reg_exp = addslashes($link_array[0][$i]);

				//ensure all characters that can be included in the text of a link are escaped
				$new_reg_exp = addcslashes($new_reg_exp, '/.)(*?:^$}{|+-}{' );

				$new_reg_exp = '/(' . $new_reg_exp . ')/';

				$text = preg_replace($new_reg_exp, $text_replace, $text);
				
			} //if
			
		} //if
		
	} //for
		
	return $text;

} //link_verifier


//check_url: given the URLs sent to this function, return the HTTP status code returned by the page corresponding to the URL
function check_url($url) {

	$session = curl_init();

	curl_setopt($session, CURLOPT_URL, $url);

	//body of the page to be returned corresponding to the URL is not needed
	curl_setopt($session, CURLOPT_NOBODY, TRUE);

	curl_setopt($session, CURLOPT_RETURNTRANSFER, TRUE);

	$download = curl_exec($session); 

	$info = curl_getinfo($session);

	$http_return_code = $info["http_code"];

	curl_close($session);

	return $http_return_code;
	

} //check_url

//only add this filter if cURL is installed
if (extension_loaded('curl')) {
	add_filter('the_editor_content', 'link_verifier');
}
	
?>